/// \file vo_fat.c FAT16/FAT32 Filesystem driver for VsOS
/// \author Panu-Kristian Poiksalo and Pasi Ojala, VLSI Solution Oy



#include <vo_stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "vsos.h"
#include "vo_fat.h"
//#include "forbid_stdout.h"
#include "strings.h"
#include <sysmemory.h>
#include <crc32.h>

#if 1
#define OPTIMIZE_SPEED
#endif

#define MIN(a,b) (((a)<(b))?(a):(b))

u_int32 VoFatCountFreeKiB(register VO_FILE *f, register u_int32 maxMiB) {
  //  FatFileInfo *fi = (FatFileInfo*)f->fileInfo;
  FatDeviceInfo *di = (FatDeviceInfo*)f->dev->deviceInfo;
  u_int32 res = 0, k;
  // totalClusters includes non-existent clusters 0 and 1
  u_int32 end = di->totalClusters;
  u_int32 maxClusters = (maxMiB == 0xFFFFFFFFU) ?
    0xFFFFFFFFU : maxMiB*(1024*2)/di->fatSectorsPerCluster;

  for (k=2; k<end; k++) {
    if (!VoFatReadClusterRecord(f, k)) {
      res++;
    }
#ifdef OPTIMIZE_SPEED
    if (di->fatBits == 32 && !(k & 127) && di->totalClusters-k >= 128) {
      u_int32 *sB = (u_int32 *)(&f->sectorBuffer[2]);
      int i;
      for (i=1; i<128; i++) {
	if (!(*sB++)) {
	  res++;
	}
      }
      k += 127;
    }
#endif

    if (res >= maxClusters) {
      return maxMiB*1024;
    }
  }

  return res * di->fatSectorsPerCluster / 2;
}
